
{% load static %}
<html lang="en"><head>
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1.0, shrink-to-fit=no">
        <title>Django Postgres Explain Visualiser Dashboard</title>
        <link href="https://cdn.jsdelivr.net/npm/@coreui/coreui@4.3.2/dist/css/coreui.min.css" rel="stylesheet" integrity="sha384-H8oVKJOQVGGCdfFNM+9gLKN0xagtq9oiNLirmijheEuqD3kItTbTvoOGgxVKqNiB" crossorigin="anonymous">
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.1/css/all.min.css" integrity="sha512-DTOQO9RWCH3ppGqcWaEA1BIZOC6xxalwEsw9c2QQeAIftl+Vegovlnee1c9QX4TctnWMn13TZye+giMm8e2LwA==" crossorigin="anonymous" referrerpolicy="no-referrer" />
        <script src="https://cdn.jsdelivr.net/npm/@coreui/coreui@4.3.2/dist/js/coreui.bundle.min.js" integrity="sha384-yaqfDd6oGMfSWamMxEH/evLG9NWG7Q5GHtcIfz8Zg1mVyx2JJ/IRPrA28UOLwAhi" crossorigin="anonymous"></script>

        <style>
            .wrapper {
                width: 100%;
                padding-left: var(--cui-sidebar-occupy-start, 0);
                will-change: auto;
                transition: padding 0.15s;
            }
        </style>

    </head>

    <body class="d-flex">
    <!-- Header content here -->
    <!-- Sidebar content here -->
        <div id="sidebar" class="sidebar sidebar-fixed">
            <div class="sidebar-brand">
                <h3> Django PEV </h3>
            </div>
            <ul class="sidebar-nav">
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:queries' %}">
                        <i class="nav-icon fa-solid fa-clipboard-question"></i> Queries
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:space' %}">
                        <i class="nav-icon fa-solid fa-chart-pie"></i> Space
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:indexes' %}">
                        <i class="nav-icon fa-solid fa-bookmark"></i> Indexes
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:live-queries' %}">
                        <i class="nav-icon fa-solid fa-play"></i> Live Queries
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:connections' %}">
                        <i class="nav-icon fa-solid fa-wifi "></i> Connections
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:explain' %}">
                        <i class="nav-icon fa-solid fa-ambulance"></i> Explain Query
                    </a>
                </li>
                <li class="nav-item">
                    <a class="nav-link" href="{% url 'django_pev:maintenance' %}">
                        <i class="nav-icon fa-solid fa-hammer"></i> Table Maintenance
                    </a>
                </li>
            </ul>
        </div>
        <div class="wrapper flex-grow-1 d-flex">
            <div class="body flex-grow-1 px-3">
                <div class="container-xl">
                {% block content %}
                    <div class="card">
                        <div class="card-body">
                        {% block inner_content %}
                        {% endblock %}
                        </div>
                    </div>
                {% endblock %}
                </div>
        <!-- Main content here -->
            </div>
        </div>

        <footer class="app-footer">
        <!-- Footer content here -->
        </footer>
    </body>
</html>